﻿init python:
    import pygame
    import random
    import time
    random.seed(time.clock())

    def load_map(tilemap):
        map = []
        f = open(renpy.loader.transfn(tilemap),"r")
        for line in f:
            line = line.strip()
            divided = [int(n) for n in line.split(",")]
            map.append(divided)
        f.close()
        return map
    def load_tiles(img, size, y = 8, x = 8):
        tiles = []
        for i in range(x):
            for ii in range(y):
                tiles.append(Crop((ii*size,i*size,size,size), img))
        return tiles


    class block:
        def __init__(self, name, image, x = 0, y = 0, hardness = 0, rotation = 0):
            self.name = name
            self.image = image
            self.x = x
            self.y = y
            self.health = 10
            self.hardness = hardness
            self.rotation = rotation
            self.accumulation = 0
        def dig(self, map, power):
            self.accumulation += power
            if power > self.hardness:
                power -= self.hardness
                self.accumulation = 0
            elif self.accumulation > self.hardness:
                self.accumulation -= self.hardness+1
                power = 1
            else:
                power = 0

            self.health -= power
            if self.health < 1:
                map.map.remove(self)
                for n,i in enumerate(map.materials):
                    if i[0] == self.name:
                        map.materials[n][1] += 1
            else:
                renpy.play("miner/punch.ogg", "sound")
            renpy.restart_interaction()

    class mine_handler:
        def __init__(self, x_in = 0, y_in = 0):
            self.materials = [
                ["Dirt",    0, 0, 60],
                ["Stone",   0, 2, 40],
                ["Coal",    0, 1, 20],
                ["Iron",    0, 1, 15],
                ["Garnet",  0, 3, 8],
                ["Gold",    0, 4, 4],
                ["Diamond", 0, 5, 2],
            ]
            mat = [ # name, index, hardness, chance
                ["Dirt",      0, 0, 60],
                ["Stone",     1, 5, 40],
                ["Coal",      2, 1, 20],
                ["Iron",      3, 4, 15],
                ["Garnet",    4, 3, 8],
                ["Gold",      5, 4, 4],
                ["Diamond",   6, 5, 2],
            ]

            self.tiles = load_tiles("16x16 dig", 16, 7, 4)
            tilemap = load_map("miner/mine_top_level.csv")

            total_chance = 0
            for i in mat:
                total_chance += i[3]
            
            map = []
            for y,j in enumerate(tilemap):
                for x,i in enumerate(j):
                    if i == 0:
                        chance = random.randint(0, total_chance+10)
                        total = 0
                        for t in mat:
                            total += t[3]
                            if chance < total:
                                map.append(block(t[0], t[1], x-x_in, y-y_in, t[2]))
                                break
                    elif i == -1:
                        pass
                    elif i in [17, 18, 19, 20, 24, 25, 26, 27]:
                        map.append(block("Stone", i, x-x_in, y-y_in, -1))
                    elif i == 15:
                        map.append(block("Stone", i, x-x_in, y-y_in, 500))
                    else:
                        map.append(block("Dirt", i, x-x_in, y-y_in, 5))
            self.map = map

            self.x = 0
            self.y = 0


    class player_sprite(renpy.Displayable):
        def __init__(self, image, map):
            super(renpy.Displayable,self).__init__()
            self.start_image = image
            self.go_left = 0
            self.go_right = 0
            self.go_up = 0
            self.go_down = 0
            self.x = 0
            self.y = 0
            self.speed = 1.0
            self.facing = 1
            self.frame = 4
            self.max_frames = 3
            self.y_velocity = 0
            self.map = map
            self.pause = 0
            self.edge_distance = 64
            self.power = 5
            self.mode = "Dig"
        def change_mode(self, mode):
            self.mode = mode
        def animate(self):
            if self.frame == 5:
                self.frame = 4
                renpy.redraw(self, 0)
            else:
                self.frame += 1
                if self.frame > self.max_frames:
                    self.frame = 0


        # def attack(self, blocks):
        #     self.frame = 5
        #     renpy.redraw(self, 0)
        #     hit = False
        #     for i in blocks:
        #         if distance(self.x, i.x*16) < 16:
        #             msg.msg("1")
        #             if distance(self.y, i.y*16) < 24:
        #                 msg.msg("2")
        #                 i.damage()
        #                 hit = True
        #                 break
        #     if hit:
        #         renpy.play("holding/punch.ogg", "sound")
        #     else:
        #         renpy.play("holding/miss.ogg", "sound")

        def render(self,width,height,st,at):
            if self.pause < 1:
                self.x += (self.go_left+self.go_right)*self.speed
                self.y += (self.go_up+self.go_down)*self.speed
                if self.x > self.edge_distance + (self.map.x*16):
                    self.map.x += 1
                    renpy.restart_interaction()
                elif self.x < -self.edge_distance + (self.map.x*16):
                    self.map.x -= 1
                    renpy.restart_interaction()
                if self.y > self.edge_distance + (self.map.y*16):
                    self.map.y += 1
                    renpy.restart_interaction()
                elif self.y < -self.edge_distance + (self.map.y*16):
                    self.map.y -= 1
                    renpy.restart_interaction()
                for i in self.map.map:
                    if not i.hardness == -1:
                        if distance(i.x, self.map.x) < 5 and distance(i.y, self.map.y) < 5:
                            if distance(self.x, i.x*16) < 12:
                                if distance(self.y, i.y*16) < 12:
                                    if self.go_left and self.x > i.x*16:
                                        self.x += 5
                                    if self.go_right and self.x < i.x*16:
                                        self.x -= 5
                                    if self.go_up and self.y > i.y*16:
                                        self.y += 5
                                    if self.go_down and self.y < i.y*16:
                                        self.y -= 5
                                    if self.mode == "Dig":
                                        i.dig(self.map, self.power)
                                    self.pause = 10
                                    break
            else:
                self.pause -= 1

            img = self.start_image

            t = Transform(img, xanchor=0.5, yanchor=0.5, rotate = self.facing)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x,self.y))

            if self.go_left or self.go_right or self.go_up or self.go_down or self.y < 0:
                renpy.redraw(self, 0)

            return rv

        def event(self, ev, x, y, st):
            if ev.type == pygame.KEYDOWN:
                if ev.key == pygame.K_a:
                    self.facing = -90
                    self.go_left = -1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_d:
                    self.facing = 90
                    self.go_right = 1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()

                if ev.key == pygame.K_w:
                    self.facing = 0
                    self.go_up = -1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_s:
                    self.facing = 180
                    self.go_down = 1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()

            elif ev.type == pygame.KEYUP:
                if ev.key == pygame.K_a:
                    self.go_left = 0
                    self.frame = 4
                    # raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_d:
                    self.go_right = 0
                    self.frame = 4
                    # raise renpy.IgnoreEvent()

                if ev.key == pygame.K_w:
                    self.go_up = 0
                    self.frame = 4
                    # raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_s:
                    self.go_down = 0
                    self.frame = 4
                    # raise renpy.IgnoreEvent()
            return None

init:
    $ config.keymap['screenshot'].append('o')

image damage 1 = Crop((0, 0, 16, 16), "damage_animation")
image damage 2 = Crop((16, 0, 16, 16), "damage_animation")
image damage 3 = Crop((32, 0, 16, 16), "damage_animation")

image damage 4 = Crop((0, 16, 16, 16), "damage_animation")
image damage 5 = Crop((16, 16, 16, 16), "damage_animation")
image damage 6 = Crop((32, 16, 16, 16), "damage_animation")

image damage 7 = Crop((0, 32, 16, 16), "damage_animation")
image damage 8 = Crop((16, 32, 16, 16), "damage_animation")
image damage 9 = Crop((32, 32, 16, 16), "damage_animation")

default mine_map = mine_handler(30, 53)
default miner = player_sprite("miner_image", mine_map)

screen mine(p = miner, g = mine_map):
    modal True
    # key "K_SPACE" action Function(g.dig)
    frame padding 0,0 xysize 336,240 align .0,.0 offset 60,60 background "#352f27":
        at zoom(4,True)
        frame padding 0,0 xysize 336,240 align .0,.0 background None:
            at mine_map_offset(-g.x*16, -g.y*16)

            for i in g.map:
                if distance(i.x, g.x) < 11 and distance(i.y, g.y) < 8:
                    fixed fit_first True:
                        align .5,.5 xysize 16,16 offset i.x*16,i.y*16
                        add g.tiles[i.image]
                        if i.health < 10:
                            add "damage {}".format(i.health)
            # add "16x16" align .5,.5
            add miner:
                align .5,.5
    add "miner_overlay"
    frame xsize 515 yfill True xalign 1.0 background None:
        hbox align .5,.0 box_wrap True spacing 54 box_wrap_spacing 54 offset 0,54:
            for i in g.materials:
                button background None:
                    xysize 96,96 padding 0,0
                    text i[0] align .5,.5
                    text str(i[1]) align 1.0,1.0
        hbox align .5,.8 spacing 10:
            for i in ["Dig", "Build", "Travel"]:
                button:
                    background "#fff6"
                    selected_background "#fff9"
                    text i
                    action Function(p.change_mode, i), SelectedIf(i == p.mode)

transform mine_map_offset(x, y):
    subpixel True
    ease .25 xoffset x yoffset y




label miner_example:
    call screen mine